## Install the lateset version of the BAS package from Github
# library(devtools)
# install_github("merliseclyde/BAS")

library(BAS); ## to fit the data
library(plotrix); ## to create the heatmaps

#####################################################
########      Read the GUSTO-I data          ########
#####################################################
#### The original data is in the "gusto" folder.
#### The .rda file of the dataset is in the "data" folder.

## Use the "foreign" library to read the .sav file
library(foreign);
gusto = read.spss('gusto/GustoW.sav', to.data.frame = TRUE);

## See Steyerberg [2009] book: Clinical Prediction Models, Ch22 for details, page 454
dim(gusto) # n = 2188, p = 26

#### clean data to be the same as Held et al [2015]
## y: DAY30
## x1: SEX
## x2: AGE
## x3: KILLIP
## x4: DIA
## x5: HYP
## x6: HRT
## x7: ANT
## x8: PMI
## x9: HEI
## x10: WEI
## x11: HTN
## x12: SMK
## x13: LIP
## x14: PAN
## x15: FAM
## x16: STE
## x17: TTR

attach(gusto);
temp = cbind(DAY30, SEX, AGE, KILLIP, DIA, HYP, HRT, ANT, PMI, HEI, WEI, HTN, SMK, LIP, PAN, FAM, STE, TTR);
detach(gusto);
colnames(temp) = c('y', paste('x', 1:17, sep = ''));
gusto = temp;
save(gusto, file = 'data/gusto.rda');

n = as.numeric(dim(gusto)[1]); p = dim(gusto)[2] - 1;
gusto = as.data.frame(gusto);

##########################################################################
#####                         Model Fitting                         ######
##########################################################################

incprob = array(NA, dim = c(18, p, 2)); ## posterior marginal inclusion probabilities
bma = array(NA, dim = c(18, p + 1, 2)); ## BMA estimates of alpha and beta
g.bma = g.map = runtime = matrix(NA, nrow = 18, ncol = 2); ## posterior estimates of g, and runtimes

## priors on the model space
modelpriors = list(uniform(), beta.binomial(1, 1)); 

## priors on beta
betapriors = list(
	CCH(a = 1 / 2, b = n, s = 0), 
	CCH(a = 1, b = n, s = 0), 
	CCH(a = 1 / 2, b = n / 2, s = 0), 
	CCH(a = 1, b = n / 2, s = 0), 
	beta.prime(n = n), 
	CCH(a = 1, b = 2, s = n + 3),
	robust(n = n), 
	intrinsic(n = n), 
	hyper.g.n(alpha = 3, n = n), 
	CCH(a = 0.02, b = 0.02 * max(n, p^2), s = 0),
	g.prior(n), 
	testBF.prior(n),
	Jeffreys(), 
	CCH(a = 1, b = 2, s = 0), 
	CCH(a = 2, b = 2, s = 0), 
	EB.local(), 
	aic.prior(), 
	bic.prior(n)
);

########################################################################
for(l in 1:2){ ## different model priors
  for(k in 1:18){ ## different betaprior
    # k = 1; l = 1;
    start.time = proc.time();
    results = bas.glm(y ~ ., data = gusto, betaprior = betapriors[[k]], method = 'BAS', n.models = 2^p, family = binomial(link = 'logit'), modelprior = modelpriors[[l]], initprobs = 'eplogp', laplace = FALSE);
    runtime[k, l] = (proc.time() - start.time)[3];

    ## BMA parameter estimation    
    bma[k, , l] = coef(results)[[1]];
    
    ## marginal includsion probability
    incprob[k, , l] = results$probne0[-1];
    
    ## posterior estimates on g
    if(k < 17){ ## not AIC or BIC
      ghat = results$shrinkage / (1 - results$shrinkage);
      g.map[k, l] = ghat[which.max(results$postprobs)];
            
      ## remove the null model when computing BMA estimate for g
      ghat.new = ghat[results$size != 1]
      postprobs = results$postprobs[results$size != 1]
      g.bma[k, l] = c(ghat.new %*% postprobs / sum(postprobs)); 
    }
    
    #save(incprob, bma, runtime, file = 'results/GUSTO_fitting_with_g.rda');
    rm(results);
  }
}            
########################################################################
####                  Heatmap: black and white                      ####
########################################################################
# load(file = 'results/GUSTO_fitting_with_g.rda');
names = c('CH(a=1/2,b=n)', 'CH(a=1,b=n)', 'CH(a=1/2,b=n/2)', 'CH(a=1,b=n/2)', 'Beta-prime', 'ZS adpated', 'Robust', 'Intrinsic', 'Hyper-g/n', 'Benchmark', 'DBF, g=n', 'TBF, g=n', 'Jeffreys', 'Hyper-g', 'Uniform', 'Local EB', 'AIC', 'BIC');
order = c(1:6, 10, 7:9, 11:18); ## The same order as shown in the paper

# pdf(file = 'figures/GUSTO-I_incprob_bw.pdf', width = 7, height = 8);
par(mar = c(4, 7, 1.5, 1), mfrow = c(2, 1), oma = c(-5, 0, 0, 0));

x = incprob[order, , 1];
color2D.matplot(x, c(1 - min(x), 1 - max(x)), c(1 - min(x), 1 - max(x)), c(1 - min(x), 1 - max(x)), main = "Uniform Prior", border = NA, axes = FALSE, xlab = '', ylab = '');
axis(1, (1:p) - 0.5, paste('X', 1:p, sep = ''), las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);
axis(2, (18:1) - 0.5, names[order], las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);

testcol = color.gradient(c(1, 0), c(1, 0), c(1, 0), nslices = 7);
color.legend(1, -4, 7, -3, c(0, 0.5, 1), testcol, align = "rb", gradient = "x");

x = incprob[order, , 2];
color2D.matplot(x, c(1 - min(x), 1 - max(x)), c(1 - min(x), 1 - max(x)), c(1 - min(x), 1 - max(x)), main = "Beta-Binomial(1,1) Prior", border = NA, axes = FALSE, xlab = '', ylab = '');
axis(1, (1:p) - 0.5, paste('X', 1:p, sep = ''), las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);
axis(2, (18:1) - 0.5, names[order], las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);
# dev.off();

########################################################################
####                  Heatmap: red and blue                         ####
########################################################################
# load(file = 'results/GUSTO_fitting_with_g.rda');
names = c('CH(a=1/2,b=n)', 'CH(a=1,b=n)', 'CH(a=1/2,b=n/2)', 'CH(a=1,b=n/2)', 'Beta-prime', 'ZS adpated', 'Robust', 'Intrinsic', 'Hyper-g/n', 'Benchmark', 'DBF, g=n', 'TBF, g=n', 'Jeffreys', 'Hyper-g', 'Uniform', 'Local EB', 'AIC', 'BIC');
order = c(1:6, 10, 7:9, 11:18); ## The same order as shown in the paper

# pdf(file = 'figures/GUSTO-I_incprob_color2.pdf', width = 7, height = 8);
par(mar = c(4, 7, 1.5, 1), mfrow = c(2, 1));
testcol = color.gradient(c(0, 0), c(0, 0), c(0, 0), nslices = 64);

x = incprob[order, , 1];
cellcol = matrix('#000000', nrow = 18, ncol = p);
cellcol[x <= 0.5] = color.scale(0.5 - x[x <= 0.5], 1 - 2 * min(0.5 - x[x <= 0.5]), c(1, 0 + 2 * min(x)), c(1, 0 + 2 * min(x)));
cellcol[x > 0.5] = color.scale(x[x > 0.5], c(1 - 2 * min(x[x > 0.5] - 0.5), 2 - 2 * max(x)), c(1 - 2 * min(x[x > 0.5] - 0.5), 2 - 2 * max(x)), 1 - 2 * min(x[x > 0.5] - 0.5));
color2D.matplot(x, cellcolors = cellcol, main = "Uniform Prior", border = 'grey', axes = FALSE, xlab = '', ylab = '');
axis(1, (1:p) - 0.5, paste('X', 1:p, sep = ''), las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);
axis(2, (18:1) - 0.5, names[order], las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);

legval = seq(0, 1, length.out = 7);
legcol = rep('#000000', 7);
legcol[legval <= 0.5] = color.scale(1 - legval[legval <= 0.5], 1, c(1, 0), c(1, 0)); 
legcol[legval >= 0.5] = color.scale(legval[legval >= 0.5], c(1, 0), c(1, 0), 1);

color.legend(1, -4, 7, -3, c(0, 0.5, 1), legcol, align = "rb", gradient = "x");

x = incprob[order, , 2];
cellcol = matrix('#000000', nrow = 18, ncol = p);
cellcol[x <= 0.5] = color.scale(0.5 - x[x <= 0.5], 1 - 2 * min(0.5 - x[x <= 0.5]), c(1, 0 + 2 * min(x)), c(1, 0 + 2 * min(x)));
cellcol[x > 0.5] = color.scale(x[x > 0.5], c(1 - 2 * min(x[x > 0.5] - 0.5), 2 - 2 * max(x)), c(1 - 2 * min(x[x > 0.5] - 0.5), 2 - 2 * max(x)), 1 - 2 * min(x[x > 0.5] - 0.5));
color2D.matplot(x, cellcolors = cellcol, main = "Beta-Binomial(1,1) Prior", border = 'grey', axes = FALSE, xlab = '', ylab = '');
axis(1, (1:p) - 0.5, paste('X', 1:p, sep = ''), las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);
axis(2, (18:1) - 0.5, names[order], las = 1, cex.axis = 1, mgp = c(0, 0.3, 0), tck = 0);
# dev.off();

########################################################################
####              Compute marginal likelihood p(Y | g)              ####
########################################################################

rm(list = ls());
load(file = 'data/gusto.rda');
n = as.numeric(dim(gusto)[1]); p = dim(gusto)[2] - 1;
y = gusto[, 1];
X = gusto[, -1];

grid.g = 1:100; ## evaluate marginal likelihood of g at a grid of values
modelg.bb = modelg.unif = matrix(NA, nrow = 2^p, ncol = length(grid.g) + p); ## output matrix, each row is a subset model
colnames(modelg.bb) = colnames(modelg.unif) = c(paste('X', 1:p, sep = ''), paste('g=', grid.g, sep = ''));

########################################################
## The first p columns of "modelg" are the model indices
for(j in 1:p)
  modelg.bb[ , j] = modelg.unif[ , j] = rep( c(rep(0, 2 ^ (p - j)), rep(1, 2 ^ (p - j))), 2 ^ (j - 1) );  	  	

## Compute p(g, M | Y) for model M, under BetaBinomial(1,1) or uniform prior over the model space
log.joint.post = function(y, X, model, grid.g){
    Xnow = as.matrix(X[ , which(model == 1)]);
  	pnow = dim(Xnow)[2];
  	
  	if(pnow == 0){
      glm1 = glm(y ~ 1, family = binomial(link = 'logit'));
      ## mle estimators
      alpha = glm1$coef[1];
      ## log maximized likelihood
      lC = sum( dbinom(y, 1, 1 / (1 + exp(-alpha)), log = TRUE) );
      ## precision of alpha
      Ieta = rep(1 / ( (1 + exp(alpha)) * (1 + exp(-alpha)) ), n);

      ## under flat prior on alpha, and Beta-Binomial(1,1) prior on models
      lpost.bb = lC - 0.5 * log(sum(Ieta)) - log(choose(length(model), pnow)); 
      ## under flat prior on alpha, and uniform prior on models
      lpost.unif = lC - 0.5 * log(sum(Ieta)) - p * log(2); 
      return(list(bb = rep(lpost.bb, length(grid.g)), unif = rep(lpost.unif, length(grid.g))));  		  		
  	}
  	
  	if(pnow > 0){
  	  glm1 = glm(y ~ Xnow, family = binomial(link = 'logit'));
      ## mle estimators
      alpha = glm1$coef[1];
      beta = glm1$coef[-1];
      ## log maximized likelihood
      lC = sum( dbinom(y, 1, 1 / (1 + exp(- alpha - Xnow %*% beta)), log = TRUE) );
      ## "centering"
      Ieta = 1 / ((1 + exp(alpha + Xnow %*% beta)) * (1 + exp(-alpha - Xnow %*% beta)));

      ## "centering"
      v = t(Ieta) %*% Xnow / sum(Ieta);
      Xc = Xnow - rep(1, n) %*% v;
      Q = sum((Xc %*% beta)^2 * Ieta);

      ## under flat prior on alpha, and Beta-Binomial(1,1) prior on models
      lpost.bb = lC - 0.5 * log(sum(Ieta)) - pnow / 2 * log(1 + grid.g) - Q / (2 * (1 + grid.g)) - log(choose(length(model), pnow)) ;  
      ## under flat prior on alpha, and uniform prior on models
      lpost.unif = lC - 0.5 * log(sum(Ieta)) - pnow / 2 * log(1 + grid.g) - Q / (2 * (1 + grid.g))  - p * log(2);  
      return(list(bb = lpost.bb, unif = lpost.unif));  		  		
      
    }
}

########################################################
for(r in 1:(2^p)){
  tmp = log.joint.post(y, X, modelg.bb[r, 1:p], grid.g);
  modelg.bb[r, -(1:p)] = tmp$bb;
  modelg.unif[r, -(1:p)] = tmp$unif;
  if(r %% (2^10) == 0)
    # save(modelg.bb, modelg.unif, file = 'results/log_modelg.rda');
}

########################################################
######  Draw the plot: marginal densities of g    ######
########################################################

par(mar = c(3, 4, 1, 1))
tmp = max(modelg.bb[, -(1:p)]);
marlik.g.bb = apply(exp(modelg.bb[, -(1:p)] - tmp), 2, sum);
plot(grid.g, marlik.g.bb / sum(marlik.g.bb), type = 'l', xlab = 'g', ylab = 'p(Y|g)');

tmp = max(modelg.unif[, -(1:p)]);
marlik.g.unif = apply(exp(modelg.unif[, -(1:p)] - tmp), 2, sum);
lines(grid.g, marlik.g.unif / sum(marlik.g.unif), col = 2, lty = 2);
legend('topright', legend = c('Uniform prior', 'Beta-Binomial(1,1) prior'), lty = 2:1, col = 2:1)


